# Copyright (c) 2003-2005 gocept gmbh & co. kg
# See also LICENSE.txt
# $Id: Install.py,v 1.9 2005/01/27 09:04:05 zagy Exp $

from Products.CMFLinkChecker.LinkCheckerTool import LinkCheckerTool
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.DirectoryView import addDirectoryViews
from Products.CMFLinkChecker import linkchecker_globals 
from Products.CMFFormController.FormAction import FormActionKey
from StringIO import StringIO
import string

from Products.CMFFormController.globalVars import ANY_BUTTON, ANY_CONTEXT


def installActionIcon(self, icon_tuple):
    ai = getToolByName(self, 'portal_actionicons')
    cat = icon_tuple[0]
    act_id = icon_tuple[1]
    try:
        ai.removeActionIcon(cat, act_id)
    except KeyError:
        # icon is not registered, ignore
        pass
    ai.addActionIcon(*icon_tuple)


def install(self):
    out=StringIO()

    if not hasattr(self, "_isPortalRoot"):
        print >> out, "Must be installed in a Plone Site"
        return
   
    print >> out, 'Installing portal_linkchecker'

    print >> out, "Registering action provider ..."
    self.portal_actions.manage_aproviders(apname="portal_linkchecker",
        add_provider=1, REQUEST=None)

    print >> out, "Adding portal tool ..."
    if 'portal_linkchecker' not in self.objectIds():
        self.manage_addProduct['CMFLinkChecker'].manage_addTool(
                                                type='CMF Linkchecker Tool')

    print >> out, "Adding catalog index ..."
    self.portal_catalog.addIndex("portal_linkchecker_uid", "FieldIndex")
    self.portal_catalog.addColumn("portal_linkchecker_uid")
    self.portal_catalog.reindexIndex('portal_linkchecker_uid', self.REQUEST)
    
    print >> out, "Adding portlet ..."
    self.left_slots = list(self.left_slots) + [
        'here/portlet_links/macros/portlet',
        'here/portlet_linksToMe/macros/portlet',
    ]

    print >> out, "Adding configlets ..."
    # Register configlet
    portal_conf=getToolByName(self,'portal_controlpanel')
    portal_conf.registerConfiglet( 'CMFLinkChecker'
                 , 'Link management'      
                 , 'string:${portal_url}/lc_configlet' 
                 , ''                   # a condition   
                 , 'Manage portal'      # access permission
                 , 'Products'           # section to which the configlet should be added: 
                                        #(Plone,Products,Members) 
                 , 1                    # visibility
                 , 'CMFLinkChecker'
                 , 'linkchecker.png' # icon in control_panel, put your own icon in the 
                                                # /skins folder of your product and change 
                                                #'site_icon.gif' to 'yourfile'
                 , 'An utility for monitoring external links in your site. Helps you to avoid dead links.'
                 , None
                 )  

    portal_conf.registerConfiglet( 'MemberLinkChecker'
                 , 'Link monitoring settings'      
                 , 'string:${portal_url}/lc_personalize_form' 
                 , ''                   # a condition   
                 , 'Set own properties'      # access permission
                 , 'Member'   
                 , 1         
                 , 'CMFLinkChecker'
                 , 'linkchecker.png' # icon in control_panel, put your own icon in the 
                                                # /skins folder of your product and change 
                                                #'site_icon.gif' to 'yourfile'
                 , 'An utility for monitoring external links in your site. Helps you to avoid dead links.'
                 , None
                 )  

    # Register skins
    print >> out, "Adding skins ..."
    addDirectoryViews(self.portal_skins, 'skins', linkchecker_globals)
    skins = self.portal_skins.getSkinSelections()
    for skin in skins:
        path = self.portal_skins.getSkinPath(skin)
        path = map(string.strip, path.split(','))
        if 'linkchecker' not in path:
            try:
                    path.insert(path.index('content'), 'linkchecker')
            except ValueError:
                    path.append('linkchecker')
            path = ','.join(path)
            self.portal_skins.addSkinSelection(skin, path)
            print >> out, skin + ",",
        else:
            print >> out, "skipped" + skin,
    print >>out, ""

    print >> out, "Adding portal properties ..."
    self._setProperty("lc_status", "", "lines")
    self.lc_status = ("red", "orange", "grey", "green",)
    self._setProperty("lc_frequency", "", "lines")
    self.lc_frequency = [str(x) for x in range(0,24,3)]
    
    print >> out, "Extending memberdata schema ..."
    memberdata = getToolByName(self, "portal_memberdata")
    memberdata._setProperty('lc_notify_details', 'lc_status', 
                            'multiple selection')
    memberdata._setProperty('lc_notify_frequency', 'lc_frequency',
                            'multiple selection')
    memberdata._setProperty('lc_notify_changes_only', '', 'boolean')
    memberdata._setProperty('lc_notify_last_notification', '2000/01/01',
                            'date')

    # Override actions for all supported default content types
    print >> out, "Registering form action hooks ..."
    pfc = getToolByName(self, 'portal_form_controller')
    pfc.addFormAction('link_edit', 'success', ANY_CONTEXT, ANY_BUTTON,
                        "traverse_to", "string:document_updatelinks")
    pfc.addFormAction('event_edit', 'success', ANY_CONTEXT, ANY_BUTTON,
                        "traverse_to", "string:document_updatelinks")
    pfc.addFormAction('document_edit', 'success', ANY_CONTEXT, ANY_BUTTON, 
                        "traverse_to", "string:document_updatelinks")
    pfc.addFormAction('newsitem_edit', 'success', ANY_CONTEXT, ANY_BUTTON, 
                        "traverse_to", "string:document_updatelinks")



    print >>out, 'Installin action icon'
    installActionIcon(self, ('plone', 'linkchecker_linksToMe',
                             'linkchecker.png', 'referenced'))


    print >> out, 'Successfully installed CMFLinkChecker'

    return out.getvalue()

def uninstall(self):
    out=StringIO()

    print >>out, "Removing configlets ..."
    portal_conf=getToolByName(self,'portal_controlpanel')
    portal_conf.unregisterConfiglet('CMFLinkChecker')
    portal_conf.unregisterConfiglet('MemberLinkChecker')

    print >>out, "Removing index ..."
    portal_cat = getToolByName(self, 'portal_catalog')
    portal_cat.delIndex('portal_linkchecker_uid')
    portal_cat.delColumn('portal_linkchecker_uid')

    print >>out, "Removing action provider registration ..."
    portal_actions = getToolByName(self, 'portal_actions')
    portal_actions.manage_aproviders(chosen=('portal_linkchecker',),
        del_provider=1)
    
    print >>out, "Removing memberdata extensions ..."
    memberdata = getToolByName(self, "portal_memberdata")
    memberdata._delProperty('lc_notify_details')
    memberdata._delProperty('lc_notify_frequency')
    memberdata._delProperty('lc_notify_changes_only')
    memberdata._delProperty('lc_notify_last_notification')

    print >>out, "Removing portal properties ..."
    self._delProperty('lc_status')
    self._delProperty('lc_frequency')

    print >>out, "Removing form_controller registrations ..."
    # Argh. The API is incomplete / awkward.
    
    keys = [FormActionKey('document_edit', 'success', ANY_CONTEXT, ANY_BUTTON, 
                            None),
            FormActionKey('event_edit', 'success', ANY_CONTEXT, ANY_BUTTON, 
                            None),
            FormActionKey('link_edit', 'success', ANY_CONTEXT, ANY_BUTTON, 
                            None),
            FormActionKey('newsitem_edit', 'success', ANY_CONTEXT, ANY_BUTTON,
                            None)]
   
    delete = self.portal_form_controller.actions.delete
    for key in keys:
        delete(key)

    return out.getvalue()
