# Copyright (c) 2003-2005 gocept gmbh & co. kg
# See also LICENSE.txt
# $Id: LinkCheckerTool.py,v 1.53 2005/01/27 09:17:31 zagy Exp $
"""CMF link checker tool"""

# Python imports
from types import StringTypes
import time

# Global Zope imports
import zLOG
from AccessControl import getSecurityManager, ClassSecurityInfo
from DateTime import DateTime
from OFS.Folder import Folder
from Globals import InitializeClass
from Acquisition import aq_base

# CMF/Plone imports
from Products.CMFCore.utils import UniqueObject, getToolByName
from Products.CMFCore.CMFCorePermissions import ManagePortal
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from Products.CMFCore.ActionProviderBase import ActionProviderBase
from Products.CMFCore.ActionInformation import ActionInformation
from Products.CMFCore.Expression import Expression

# Sibling imports
from Products.CMFLinkChecker.interfaces import ILinkManager, IUIDManager
from Products.CMFLinkChecker.LinkInfo import LinkInfo
from Products.CMFLinkChecker import permissions
from Products.CMFLinkChecker.utils import resolveRelativeLink
from Products.CMFLinkChecker import shorturls

class LinkCheckerTool(UniqueObject, ActionProviderBase, Folder):
    """A link checker tool. Encapsulating mechanisms for finding links in the
       site, testing their availability and notifying their owners."""

    __implements__ = (ILinkManager, IUIDManager)

    id = 'portal_linkchecker'
    meta_type = 'CMF Linkchecker Tool'

    security = ClassSecurityInfo()

    plone_tool = 1

    manage_options = ( ({'label' : 'Overview', 'action' : 'manage_overview'},) +
                        ActionProviderBase.manage_options +
                        Folder.manage_options )

    _actions = [ActionInformation(id='linkchecker_object_status',
                                  title='Links',
                                  action=Expression(text='string:${object_url}/lc_object_status'),
                                  permissions=('Modify portal content',),
                                  category='object',
                                  visible=1),
                ActionInformation(id='linkchecker_member_overview',
                                 title='My Links',
                                 action=Expression(text='string:${portal_url}/lc_my_dead_links'),
                                 condition='member',
                                 category='user',
                                 visible=1),
                ActionInformation(id='linkchecker_balanced_scorecard',
                                 title='Link management overview',
                                 category='portal_tabs',
                                 permissions=('Manage portal',),
                                 visible=1,
                                 action=Expression(text='string:${portal_url}/lc_balanced_score_card')),
                ActionInformation(id='linkchecker_linksToMe',
                                  title='Object is being referenced',
                                  category='folder_contents',
                                  permissions=(permissions.USE_LINK_MANAGEMENT, ),
                                  visible=1,
                                  condition='python: object.lc_linksToMe()',
                                  action=Expression(text='python:None')),
                ]
                
    #
    # ZMI methods
    #
    security.declareProtected(ManagePortal, 'manage_overview')
    manage_overview = PageTemplateFile('www/explainLinkcheckerTool', globals(),
                                        __name__='manage_overview')

    def manage_afterAdd(self, container, object):
        # Create retriever manager
        self.manage_addProduct["CMFLinkChecker"].manage_addRetrieveManager('retrieving')
        # Create database
        self.manage_addProduct["CMFLinkChecker"].manage_addLinkDatabase('database')
        # Create report tool
        self.manage_addProduct["CMFLinkChecker"].manage_addBaseReports('reports')

    ##############
    # ILinkManager

    
    security.declarePublic("shortURL")
    def shortURL(self, url, target=50):
        """Context aware wrapper for creating a shortened URL"""
        base = self.database.defaultURLPrefix
        return shorturls.shortURL(url, target, base)
        
    security.declarePublic("isManaged")
    def isManaged(self, object):
        """Tells if the object is managed by the link manager."""
        return self.retrieving.supportsRetrieving(object)

    security.declarePublic('resolveRelativeLink')
    def resolveRelativeLink(self, url, context):
        return resolveRelativeLink(url, context)

    def isUserAllowed(self):
        """returns whether the user is allowed to use Link Management services
        """
        security = getSecurityManager()
        portal = getToolByName(self, 'portal_url').getPortalObject()
        return security.checkPermission(permissions.USE_LINK_MANAGEMENT,
                                        portal)
        

    security.declarePublic('getLinkManager')
    def getLinkManager(self):
        return self.aq_inner
        

    #############
    # IUIDManager

    security.declarePrivate('getUIDForObject')
    def getUIDForObject(self, object):
        """see IUIDManager"""
        if object is None:
            return None

        uid = None

        unwrapped = aq_base(object)
        uid = getattr(unwrapped, 'portal_linkchecker_uid', None)

        if uid is None:
            uid = self.generateUniqueId(object.getId())
            object.portal_linkchecker_uid = uid
            pc = getToolByName(self, "portal_catalog")
            pc.reindexObject(object, idxs=['portal_linkchecker_uid'])

        return uid

    security.declarePrivate('getUIDForBrain')
    def getUIDForBrain(self, brain):
        """see IUIDManager"""
        return getattr(brain, 'portal_linkchecker_uid', None)

    security.declarePublic('getObjectForUID')
    def getObjectForUID(self, uid):
        """Return the object corresponding to this UID.

           Raises KeyError if the UID doesn't correspond to an object.

           Returns an object.
        """
        results = self.portal_catalog(portal_linkchecker_uid=uid)
        if not results:
                raise KeyError, "No object with uid %s is registered" % uid
        if len(results) > 1:
                raise ValueError, "Object is not uniquely identified by %s" \
                    "(%s alternatives)." % (uid, len(results))

        return results[0].getObject()

InitializeClass(LinkCheckerTool)
