# Copyright (c) 2003-2005 gocept gmbh & co. kg
# See also LICENSE.txt
# $Id: LinkInfo.py,v 1.13 2005/01/27 09:17:41 zagy Exp $
"""CMF link checker tool - link information object
"""

# Zope imports
from DateTime import DateTime
from OFS.SimpleItem import SimpleItem
from Products.PageTemplates.PageTemplateFile import PageTemplateFile

# CMF/Plone imports
from Products.CMFCore.utils import getToolByName

# Sibling imports
from Products.CMFLinkChecker.interfaces import ILinkInformation


class LinkInfo(SimpleItem):

    meta_type = "Link Information"

    __implements__ = (ILinkInformation, )
    __allow_access_to_unprotected_subobjects__ = 1      # XXX make accessor functions

    objects = []
    url = None
    lastcheck = None
    lastupdate = None
    laststatus = "grey"
    state = "grey"
    reason = ""
    registered = False

    manage_options = ({'label': 'Info', 'action':'manage_info'},) + \
                     SimpleItem.manage_options

    manage_info = PageTemplateFile('www/linkinfo', globals(), __name__='manage_info')

    def __init__(self, url, id):
        self.objects = []
        self.url = url
        self.id = id

    def __repr__(self):
        return "<LinkInfo %s (%s)>" % (self.url, self.state)

    def _p_resolveConflict(self, oldState, savedState, newState):
        return newState

    def getId(self):
        return self.id

    def title(self):
        return self.url
        
    def addObject(self, object):
        """Add this object to the object list."""
        uid = self.portal_linkchecker.getUIDForObject(object)
        if not uid in self.objects:
            self.objects.append(uid)
            self._p_changed = 1
            self.index()
        
    def removeObject(self, object):
        """Remove this uid from the object list."""
        uid = self.portal_linkchecker.getUIDForObject(object)
        if uid in self.objects:
            self.objects.remove(uid)
            self._p_changed = 1
            self.index()
        else:
            raise KeyError, object

    def clearObjects(self):
        """Remove all uid references from the object list."""
        self.objects = []
        self._p_changed = 1
        self.index()

    def getObjects(self):
        """Return a reference to the object."""
        objects = [ self.portal_linkchecker.getObjectForUID(object) 
                    for object in self.objects ]
        objects = [ x for x in objects if x is not None ]
        return objects

    def updateStatus(self, state, reason):
        assert state in ['red', 'green', 'orange', 'grey'], "Invalid state %s" % state
        self.reason = reason
        if state != self.state:
            self.laststate = self.state
            self.state = state 
            self.lastupdate = DateTime()
        self.index()

    def getGroupId(self):
        return self.state

    # Catalog support

    def manage_afterAdd(self, container, object):
        self.index()

    def manage_beforeDelete(self, item, container):
        self.unindex()

    def index(self):
        self.link_catalog.catalog_object(self, '/'.join(self.getPhysicalPath()))

    def unindex(self):
        self.link_catalog.uncatalog_object('/'.join(self.getPhysicalPath()))



class NullLinkInfo:

    objects = []
    url = "no"
    lastcheck = None
    lastupdate = None
    laststate = "grey"
    state = "grey"
    reason = "I'm stupid"
    queued = 0
    
    def addObject(object):
        pass

    def removeObject(object):
        pass

    def getObjects():
        return []

    def updateStatus(state, reason):
        pass

