##############################################################################
#
# Copyright (c) 2003 gocept gmbh & co. kg. All rights reserved.
#
# See also LICENSE.txt
#
##############################################################################
"""CMF link checker tool - link retriever functions

$Id: retrievers.py,v 1.15 2004/09/17 13:25:28 ctheune Exp $"""

# CMFLinkChecker imports
from Products.CMFLinkChecker.interfaces import IRetriever
from Products.CMFLinkChecker.utils import retrieveHTML, retrieveSTX
from Products.CMFLinkChecker.retrievemanager import GlobalRegistry

class RetrieverLink(object):
    """CMFDefault.Link retriever"""

    __implements__ = (IRetriever,)

    name = "CMFDefault.Link"
    defaults = ["Link",]

    def __call__(self, object):
        try:
            return [object.getRemoteUrl()]
        except AttributeError:  # Incompatible content selected ... XXX report warning here
            return []

GlobalRegistry.register(RetrieverLink())
    
class RetrieverEvent(object):
    """CMFDefault.Event retriever"""

    __implements__ = (IRetriever,)

    name = "CMFDefault.Event"
    defaults = ["Event",]
    
    def __call__(self, object):
        try:
            links = [object.event_url]
        except AttributeError:  # Incompatible content selected ... XXX report warning here
            links = []
        return links

GlobalRegistry.register(RetrieverEvent())

class RetrieverDocument(object):
    """CMFDefault.Document retriever"""

    __implements__ = (IRetriever,)

    name = "CMFDefault.Document"
    defaults = ["Document", "News Item"]

    def __call__(self, object):
        if not hasattr(object, "text_format"):  # XXX report warning here
            return 
        # Handle document modes
        if object.text_format == 'structured-text':
            links = retrieveSTX(object.text)
        elif object.text_format == 'html':
            links = retrieveHTML(object.text)
        else:
            links = []
        return links

GlobalRegistry.register(RetrieverDocument())

