# Copyright (c) 2004 gocept gmbh & co. kg
# See also LICENSE.txt
# $Id: test_linkchecker.py,v 1.5 2005/01/27 09:17:14 zagy Exp $

import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

import unittest
import time, Queue

from Testing import ZopeTestCase

from Products.Archetypes.tests.common import *
from Products.Archetypes.tests.utils import *

from Products.CMFLinkChecker.tests.LinkCheckerTestCase import \
    LinkCheckerTestCase

from Products.Archetypes.tests.test_sitepolicy import makeContent
from Products.CMFCore.utils import getToolByName

from Products.CMFLinkChecker.interfaces import ILinkInformation

class CMFLinkCheckerTest(LinkCheckerTestCase):

    def test_link_registration(self):
        doc = makeContent(self.getPortal(), portal_type="Document", id="asdf")

        lc = getToolByName(self.getPortal(), 'portal_linkchecker')
        links = ["http://www.gocept.com", 
                 "http://www.asfdhasdkjfhasdkjhfkjsda.org", 
                 "http://www.zope.org", "http://www.microsoft.com"]

        for link in links:
            info = lc.database.registerLink(link, doc)

        # Check for the links the object was registered for
        doc_links = lc.database.byObject(doc)
        self.assertEqual(len(links), len(doc_links))

        for l in doc_links:
            self.failUnless(l.url in links)

    def test_crawling(self):
        lc = getToolByName(self.getPortal(), 'portal_linkchecker')
        
        links = ["http://www.google.de", "http://www.gocept.com", 
            "http://www.arcor.de", "http://www.gocept.com"]
        docs = []
        
        for x in links:
            id = self.getPortal().generateUniqueId()
            doc = makeContent(self.getPortal(), portal_type="Document", id=id)
            doc.edit('html', '<a href="%s">asdf</a>' % x)
            docs.append(doc)

        lc.retrieving.retrieveSite()

        self.assertEqual(lc.database.getLinkCount(), 3)    # Doubles only counted once

        for doc in docs:
            l = lc.database.byObject(doc)
            self.assertEqual(len(l), 1)
            self.failUnless(ILinkInformation.isImplementedBy(l[0]))
            self.failUnless(l[0].url in links)

    def test_byobject(self):
        lc = getToolByName(self.getPortal(), 'portal_linkchecker')
        db = lc.database
        doc = makeContent(self.getPortal(), portal_type="Document", id="asdf")
        doc.text = '''"blubb":bla/fasel
            "gocept":http://www.gocept.com'''
        lc.retrieving.retrieveObject(doc)

        links = db.byObject(doc)
        self.assertEquals(2, len(links))

        red_link = links[0]
        grey_link = links[1]
        red_link.updateStatus('red', 'testing')
        grey_link.updateStatus('grey', 'testing')
        
        links = db.byObject(doc, state='red')
        self.assertEquals(1, len(links))

        links = db.byObject(doc, state=['red', 'grey'])
        self.assertEquals(2, len(links))


        


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(CMFLinkCheckerTest))
    return suite

if __name__ == '__main__':
    framework()

