# Copyright (c) 2004 gocept gmbh & co. kg
# See also LICENSE.txt
# $Id: test_retrievers.py,v 1.2 2004/09/17 13:25:29 ctheune Exp $

import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

import unittest

from Testing import ZopeTestCase
ZopeTestCase.installProduct("CMFLinkChecker")

from Products.Archetypes.tests.common import *
from Products.Archetypes.tests.utils import *

from Products.Archetypes.tests.test_sitepolicy import makeContent
from Products.CMFCore.utils import getToolByName

from Products.CMFLinkChecker.retrievers import RetrieverEvent, RetrieverLink, \
        RetrieverDocument

class CMFLinkCheckerTest(ArcheSiteTestCase):

    def afterSetUp(self):
        ArcheSiteTestCase.afterSetUp(self)

        user = self.getManagerUser()
        newSecurityManager(None, user)

        portal = self.getPortal()
        qi = getToolByName(portal, 'portal_quickinstaller')
        qi.installProduct('CMFLinkChecker')

    def test_event_retriever(self):
        lc = getToolByName(self.getPortal(), 'portal_linkchecker')

        url = "http://asdf.org"
        event = makeContent(self.getPortal(), portal_type="Event", id="asdf")
        event.edit(event_url=url)

        links = RetrieverEvent()(event)

        self.assertEqual(len(links), 1)
        self.assertEqual(links[0], url)

    def test_link_retriever(self):
        lc = getToolByName(self.getPortal(), 'portal_linkchecker')

        url = "http://www.asdf.org/asdf?asdf"
        ob = makeContent(self.getPortal(), portal_type="Link", id="asdf")
        ob.edit(url)

        links = RetrieverLink()(ob)

        self.assertEqual(len(links), 1)
        self.assertEqual(links[0], url)

    def test_document_html_retriever(self):
        links = ["http://www.asdf.org", "http://www.bauhaus.de", "ftp://www.goodbye.de", "http://www.google.de/img.png"]

        text = " ".join( [ """<a href="%s">asdf</a>""" % x for x in links[:-1] ])
        text += """<img src="%s"/>""" % links[-1]

        lc = getToolByName(self.getPortal(), 'portal_linkchecker')

        url = "http://www.asdf.org/asdf?asdf"
        ob = makeContent(self.getPortal(), portal_type="Document", id="asdf")
        ob.edit("html", text)

        links_ob = RetrieverDocument()(ob)

        self.assertEqual(len(links), len(links_ob))
        for x in links_ob:
            self.assert_(x in links)

    def test_document_stx_retriever(self):
        links = ["http://www.asdf.org", "http://www.bauhaus.de", "ftp://www.goodbye.de", "http://www.google.de/img.png"]

        text = " ".join( [ """"asdf":%s\n""" % x for x in links ])

        lc = getToolByName(self.getPortal(), 'portal_linkchecker')

        url = "http://www.asdf.org/asdf?asdf"
        ob = makeContent(self.getPortal(), portal_type="Document", id="asdf")
        ob.edit("stx", text)

        links_ob = RetrieverDocument()(ob)

        self.assertEqual(len(links), len(links_ob))
        for x in links_ob:
            self.assert_(x in links)

def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(CMFLinkCheckerTest))
    return suite

if __name__ == '__main__':
    framework()
