##############################################################################
#
# Copyright (c) 2003 gocept gmbh & co. kg. All rights reserved.
#
# See also LICENSE.txt
#
##############################################################################
"""CMF link checker tool - general utility functions

$Id: utils.py,v 1.6 2005/01/27 07:45:35 zagy Exp $"""

# CMF/Plone imports
from Products.CMFCore.utils import getToolByName

# Python imports
from urlparse import urlparse, urlunparse
from sgmllib import SGMLParseError
import formatter

# Zope imports
from StructuredText import Basic
from StructuredText.DocumentClass import StructuredTextLink, DocumentClass

# Sibling imports
from Products.CMFLinkChecker import lchtmllib

def url2dict(url):
    x = {}
    (x['scheme'], x['netloc'], x['path'], x['parameters'], x['query'], \
            x['fragment']) = urlparse(url)
    return x

def resolveRelativeLink(url, context):
    """Tries to canonize the given URL"""

    # Don't canonize mailto: links (maybe some others in future as well)
    url_ = url2dict(url)

    if url_['scheme'] in ['mailto',]:
        return url

    lc = getToolByName(context, "portal_linkchecker")
    prefix = lc.database.defaultURLPrefix or context.REQUEST.BASE0
    prefix_dict = url2dict(prefix)
        
    new_url = ""
    
    if url_['scheme']:
        # Url has a protocol specified, i.e. http://
        new_url = url
    elif url_['netloc']:
        # There is no protocol but a host, i.e. "gocept.com/index.html"
        # We just assume http in this case
        new_url = "http://" + url
    elif url_['path'].startswith('/'):
        # The only thing we have is an absolute path. We prepend the Protocol
        # and host from the configured prefix (or use request if nothing has
        # been specified)
        new_url = '%s://%s%s' % (prefix_dict['scheme'],
                                 prefix_dict['netloc'],
                                 url)
    else:
        # This path seems to be relative. 
        context_path = '/'.join(context.getPhysicalPath()[:-1])
        new_url = '%s%s/%s' % (prefix, context_path, url)

    new_url = stripRelativeFromUrl(new_url)
    return new_url


def stripRelativeFromUrl(url):
    # noop so far
    url_split = list(urlparse(url))
    
    path = url_split[2]
    path = path.split('/')
    path.reverse()
    
    result_path = []
    skip = 0
    
    for element in path:
        if element == '..':
            skip += 1
            continue
        if skip > 0:
            skip -= 1
            continue
        result_path.append(element)
    
    result_path.reverse()
    path = '/'.join(result_path)
    url_split[2] = path
    url = urlunparse(url_split)
    return url


def retrieveSTX(text):
    """Retrieve links from STX data."""
    data = Basic(text)
    data = DocumentClass()(data)
    
    stack = data.getChildren()
    links = []
    while stack:
        element = stack.pop()
        if isinstance(element, StructuredTextLink):
            # Handle images seperately
            element = element.href
            if element.startswith('img:'):
                element = element[4:]
            links.append(element)
        try:
            stack.extend(element.getChildren())
        except AttributeError:
            pass

    return links

def retrieveHTML(text):
    parser = lchtmllib.LCHTMLParser(formatter.NullFormatter())
    try:
        parser.feed(text)
    except SGMLParseError:
        # SGMLLib seems to die on bad HTML sometimes. (At least with python2.1)
        pass
    parser.close()
    return tuple(parser.anchorlist) + tuple(parser.imglist)
